import 'package:flutter/material.dart';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:outline_material_icons/outline_material_icons.dart';

import 'package:spartathlon_app/AppStateContainer.dart';
import 'package:spartathlon_app/models/Athlete.dart';
import 'package:spartathlon_app/providers/AthleteProvider.dart';
import 'package:spartathlon_app/util/Utils.dart';

/// Athletes View
///
/// Displays Material cards giving details about a specific athlete such as
/// nationality, date of birth, their biography and list of finishes

final double elevation = 4.0;
final double pad = 12.0;

class AthleteDetailsView extends StatefulWidget {
  AthleteDetailsView({Key key, this.athlete}) : super(key: key);

  final Athlete athlete;

  @override
  _AthleteDetailsViewState createState() => _AthleteDetailsViewState();
}

class _AthleteDetailsViewState extends State<AthleteDetailsView> {
  @override
  Widget build(BuildContext context) {
    double width = MediaQuery.of(context).size.width;

    return CustomScrollView(
      slivers: [
        SliverAppBar(
          backgroundColor: Theme.of(context).primaryColor,
          expandedHeight: width,
          floating: false,
          pinned: true,
          actions: [
            FutureBuilder<bool>(
              future: widget.athlete.isFavorite(),
              builder: (BuildContext context, AsyncSnapshot<bool> snapshot) {
                if (snapshot.hasData) {
                  return IconButton(
                      icon: (snapshot.data)
                          ? Icon(
                              Icons.star,
                              color: Colors.white,
                            )
                          : Icon(
                              Icons.star_border,
                              color: Colors.white,
                            ),
                      onPressed: () {
                        setState(() {
                          AppStateContainer.of(context)
                              .setToggleFavoriteAthlete(widget.athlete.id);
                        });
                      });
                } else {
                  return Container();
                }
              },
            ),
          ],
          flexibleSpace: FlexibleSpaceBar(
              // TODO align title text to the far left
              // tracked in https://github.com/flutter/flutter/issues/11951
              title: Text(widget.athlete.name),
              background: Stack(
                fit: StackFit.expand,
                children: [
                  CachedNetworkImage(
                    imageUrl: widget.athlete.imageUrl,
                    fit: BoxFit.cover,
                    alignment: Alignment(0, 0.5),
                  ),
                  GestureDetector(onTap: () {
                    showDialog(
                      context: context,
                      builder: (BuildContext context) {
                        return SimpleDialog(
                          shape: RoundedRectangleBorder(
                              borderRadius: BorderRadius.circular(16.0)),
                          contentPadding: EdgeInsets.all(8.0),
                          children: [
                            ClipRRect(
                              borderRadius: BorderRadius.circular(10.0),
                              child: CachedNetworkImage(
                                imageUrl: widget.athlete.imageUrl,
                                fit: BoxFit.contain,
                              ),
                            )
                          ],
                        );
                      },
                    );
                  })
                ],
              )),
        ),
        SliverList(
          delegate: SliverChildListDelegate(
            [
              FutureBuilder(
                  future: AthleteProvider.getFullAthleteDetails(widget.athlete),
                  builder:
                      (BuildContext context, AsyncSnapshot<Athlete> snapshot) {
                    if (snapshot.hasData) {
                      Athlete a = snapshot.data;
                      return Container(
                        color: Theme.of(context).scaffoldBackgroundColor,
                        constraints: BoxConstraints(
                            minHeight: MediaQuery.of(context).size.height - 80),
                        child: Column(
                          children: [
                            _getNationBirthdayCard(a),
                            _getBiographyCard(a),
                            _getFinishesCard(a),
                          ],
                        ),
                      );
                    } else {
                      return Container(
                        height: MediaQuery.of(context).size.height / 2.27,
                        color: Theme.of(context).scaffoldBackgroundColor,
                        child: Center(
                          child: CircularProgressIndicator(),
                        ),
                      );
                    }
                  }),
            ],
          ),
        ),
      ],
    );
  }

  /// Return a card with the athletes nationality and birthday
  Widget _getNationBirthdayCard(Athlete a) {
    return Container(
      color: Theme.of(context).scaffoldBackgroundColor,
      padding: EdgeInsets.all(pad),
      child: Card(
        shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(Utils.cornerRadius)),
        elevation: elevation,
        child: Column(
          children: [
            ListTile(
              leading: Icon(
                Icons.public,
                color: Theme.of(context).primaryColor,
              ),
              subtitle: Text('Country'),
              title: Text(a.nation),
            ),
            Divider(height: 0.0),
            Divider(height: 0.0),
            ListTile(
              leading: Icon(
                OMIcons.cake,
                color: Theme.of(context).primaryColor,
              ),
              subtitle: Text('Date of birth'),
              title: Text(a.dateOfBirth),
            ),
          ],
        ),
      ),
    );
  }

  /// Returns a Material card with the athletes biography (aka Mini CV)
  Widget _getBiographyCard(Athlete a) {
    if (a.biography.isNotEmpty) {
      return Container(
        color: Theme.of(context).scaffoldBackgroundColor,
        padding: EdgeInsets.only(right: pad, left: pad, bottom: pad),
        child: Card(
          shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(Utils.cornerRadius)),
          elevation: elevation,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              ListTile(
                //leading: new Icon(Icons.info),
                title: Text(
                  'Biography',
                  style: TextStyle(color: Theme.of(context).primaryColor),
                ),
              ),
              Divider(height: 0.0),
              Divider(height: 0.0),
              Container(
                padding: EdgeInsets.all(16.0),
                child: Text(a.biography),
              ),
            ],
          ),
        ),
      );
    } else {
      return Container();
    }
  }

  /// Returns a Material card with a list of finishes for athlete a
  Widget _getFinishesCard(Athlete a) {
    return Container(
      color: Theme.of(context).scaffoldBackgroundColor,
      padding: EdgeInsets.only(right: pad, left: pad, bottom: pad),
      child: Card(
        shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(Utils.cornerRadius)),
        color: Theme.of(context).cardColor,
        elevation: elevation,
        child: Column(
          children: _getFinishTiles(a),
        ),
      ),
    );
  }

  /// Return a Material Card with a list of finishes
  List<Widget> _getFinishTiles(Athlete a) {
    List<Widget> list = [
      Container(
        padding: (a.finishes.isNotEmpty)
            ? EdgeInsets.only(top: 16.0, left: 16.0, right: 16.0, bottom: 0.0)
            : EdgeInsets.only(top: 16.0, left: 16.0, right: 16.0, bottom: 10.0),
        // For some reason these need to be different by 1 otherwise Flutter
        // will draw a black line of height 1 between the bio and the finishes card
        child: Row(
          children: [
            Expanded(
              child: Text(
                'Finishes',
                style: TextStyle(
                    color: Theme.of(context).primaryColor, fontSize: 16.0),
              ),
            ),
            Text(
              a.finishes.length.toString(),
              style: TextStyle(
                  color: Theme.of(context).primaryColor, fontSize: 16.0),
            ),
          ],
        ),
      ),
    ];
    if (a.finishes.isNotEmpty) {
      List<Widget> finishListHeaderItemList = [
        _CustomFinishTile(Finish(year: 'Year', time: 'Time', bib: 'Bib Nr.')),
        Divider(height: 0.0),
        Divider(height: 0.0),
      ];
      List<_CustomFinishTile> finishTiles =
          List<Finish>.from(a.finishes).map((Finish f) {
        return _CustomFinishTile(f);
      }).toList();

      list.addAll(finishListHeaderItemList);
      list.addAll(finishTiles);
    }
    list.add(Container(height: 8.0)); // Bottom padding
    return list;
  }
}

/// Defines a custom tile, similar to a LisTile, but so that "leading", "title",
/// and "trailing" all use the same font size.
class _CustomFinishTile extends StatelessWidget {
  _CustomFinishTile(this.f, {Key key}) : super(key: key);

  final Finish f;

  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
        color: Theme.of(context).cardColor,
        borderRadius: BorderRadius.circular(4.0),
      ),
      padding: EdgeInsets.only(top: 12.0, left: 16.0, right: 16.0, bottom: 8.0),
      child: Row(
        children: [
          Container(
            width: 50.0,
            child: Text(f.year),
          ),
          Expanded(
            child: Container(
              child: Text(f.time),
            ),
          ),
          Container(
            child: Text(f.bib),
          ),
        ],
      ),
    );
  }
}
